import OpenEXR
import Imath
from PIL import Image, ImageMath 
import sys
import os
from shutil import copyfile
import numpy as np
import cv2
import scipy.io as scio

def exrToMatCol(exrFilePath, isNormal):
    file = OpenEXR.InputFile(exrFilePath)
    pt = Imath.PixelType(Imath.PixelType.FLOAT)
    dw = file.header()['dataWindow']
    size = (dw.max.x - dw.min.x + 1, dw.max.y - dw.min.y + 1)
	
    if(isNormal):
         [r, g, b] = [Image.frombytes("F", size, file.channel("X", pt)), Image.frombytes("F", size, file.channel("Y", pt)) , Image.frombytes("F", size, file.channel("Z", pt))]   
    else: 
         [r, g, b] = [Image.frombytes("F", size, file.channel("R", pt)), Image.frombytes("F", size, file.channel("G", pt)) , Image.frombytes("F", size, file.channel("B", pt))] 
   
    arrR = np.asarray(r) 
    arrG = np.asarray(g)
    arrB = np.asarray(b)
	
    merge = np.dstack((arrR, arrG, arrB)) #np.concatenate([arrR, arrG, arrB])
    
	
    fileNameLong   = os.path.basename(exrFilePath)
    fileNameShort  = os.path.splitext(fileNameLong)[0]
	
    dirname    = os.path.dirname(exrFilePath)
    scio.savemat(dirname + '/' + fileNameShort + '.mat', {'data': merge})

def exrToMatGray(exrFilePath):
    file = OpenEXR.InputFile(exrFilePath)
    pt = Imath.PixelType(Imath.PixelType.FLOAT)
    dw = file.header()['dataWindow']
    size = (dw.max.x - dw.min.x + 1, dw.max.y - dw.min.y + 1)

    dep_img = Image.frombytes("F", size, file.channel("Z", pt))
   
    arrZ = np.asarray(dep_img) 
	
    fileNameLong   = os.path.basename(exrFilePath)
    fileNameShort  = os.path.splitext(fileNameLong)[0]
	
    dirname    = os.path.dirname(exrFilePath)
    scio.savemat(dirname + '/' + fileNameShort + '.mat', {'data': arrZ})	

def exrToLdrCol(exrFilePath, ldrFilePath, isNormal):
    file = OpenEXR.InputFile(exrFilePath)
    pt = Imath.PixelType(Imath.PixelType.FLOAT)
    dw = file.header()['dataWindow']
    size = (dw.max.x - dw.min.x + 1, dw.max.y - dw.min.y + 1)
    
    if(isNormal):
         rgbf = [Image.frombytes("F", size, file.channel(c, pt)) for c in "XYZ"]
    else: 
         rgbf = [Image.frombytes("F", size, file.channel(c, pt)) for c in "RGB"]
	
    extrema = [im.getextrema() for im in rgbf]
    darkest = min([lo for (lo,hi) in extrema])
    lighest = max([hi for (lo,hi) in extrema])
    scale = 255 / (lighest - darkest)

    def normalize_0_255(v):
        return (v * scale ) + darkest
    rgb8 = [im.point(normalize_0_255).convert("L") for im in rgbf]
    Image.merge("RGB", rgb8).save(ldrFilePath)	
	
    '''
    if(isNormal):
         [r, g, b] = [Image.frombytes("F", size, file.channel("Y", pt)), Image.frombytes("F", size, file.channel("X", pt)) , Image.frombytes("F", size, file.channel("Z", pt))]   
    else: 
         [r, g, b] = [Image.frombytes("F", size, file.channel("R", pt)), Image.frombytes("F", size, file.channel("G", pt)) , Image.frombytes("F", size, file.channel("B", pt))] 
    
	
	# saving the image as opencv image, good for LDR conversion 
    arrR = np.asarray(r) 
    arrG = np.asarray(g)
    arrB = np.asarray(b)

    img_bgr = np.zeros([300, 300, 3])
	
    img_bgr[:,:,0] = arrB # cv2.merge([arrB,arrG,arrR])
    img_bgr[:,:,1] = arrG
    img_bgr[:,:,2] = arrR
	
    #img = cv2.cvtColor(img_rgb, cv2.COLOR_RGB2BGR)
    cv2.imwrite(ldrFilePath, img_bgr)
	'''

def exrToLdrGray(exrFilePath, ldrFilePath):
    file = OpenEXR.InputFile(exrFilePath)
    pt = Imath.PixelType(Imath.PixelType.FLOAT)
    dw = file.header()['dataWindow']
    size = (dw.max.x - dw.min.x + 1, dw.max.y - dw.min.y + 1)

    rgbf = Image.frombytes("F", size, file.channel("Z", pt)) 

    extrema = [rgbf.getextrema()]
    darkest = min([lo for (lo,hi) in extrema])
    lighest = max([hi for (lo,hi) in extrema])
    scale = 255 / (lighest - darkest)
	
    def normalize_0_255(v):
        return (v * scale) + darkest
    rgb8 = rgbf.point(normalize_0_255).convert("L")
    rgb8.save(ldrFilePath)

# initialize root paths
scene_root = '../Rendering_Scripts/Rendered_Data/Living_Room' # path where the rendered data is stored, this can be generated using the rendering scripts  


render_root = scene_root + '/Rendered'
process_root     = scene_root + '/processed'

#create a directory which contains the .exr version of all renderings 
exr_path = process_root + '/HDR'
if not os.path.exists(exr_path):
    os.makedirs(exr_path)
	

orig_ldr_dir  = process_root + '/LDR/orig'
diff_ldr_dir  = process_root + '/LDR/diff'

shad_dir_ldr_dir  = process_root + '/LDR/shad_dir'
shad_ind_ldr_dir  = process_root + '/LDR/shad_ind'

refl_ldr_dir  = process_root + '/LDR/refl'
shad_ldr_dir  = process_root + '/LDR/shad'



spec_ldr_dir  = process_root + '/LDR/specl'

depth_ldr_dir  = process_root + '/LDR/depth'
normal_ldr_dir = process_root + '/LDR/normal'


if not os.path.exists(orig_ldr_dir):
    os.makedirs(orig_ldr_dir)
	
if not os.path.exists(diff_ldr_dir):
    os.makedirs(diff_ldr_dir)

if not os.path.exists(refl_ldr_dir):
    os.makedirs(refl_ldr_dir)

if not os.path.exists(shad_ldr_dir):
    os.makedirs(shad_ldr_dir)

if not os.path.exists(spec_ldr_dir):
    os.makedirs(spec_ldr_dir)

if not os.path.exists(depth_ldr_dir):
    os.makedirs(depth_ldr_dir)
	
if not os.path.exists(normal_ldr_dir):
    os.makedirs(normal_ldr_dir)	
	
if not os.path.exists(shad_dir_ldr_dir):
    os.makedirs(shad_dir_ldr_dir)

if not os.path.exists(shad_ind_ldr_dir):
    os.makedirs(shad_ind_ldr_dir)		

#debug check-point	
#print diff_render_root
#print spec_render_root

#traverse through all rendered images for both diffuse and original version 
for filename in os.listdir(render_root):
     file = os.path.splitext(filename)[0]
     exr_file_path = exr_path + '/' + file
	 
	 #create a directory where we store the original, diffuse, depth and reflectance images in .exr format
     if not os.path.exists(exr_file_path):
          os.makedirs(exr_file_path)
     
	 #copying the multi-layer exr file to the respective view sub-folder in the EXR folder 
     srcRender  = render_root + '/' + filename
	 
     dstRender  = exr_file_path + '/render.exr'
     
     copyfile(srcRender, dstRender)	
     
	 # splitting the multi-layer diffuse rendering to individual layered images
     os.system('cd '+  exr_file_path +';'+ 'python -m exrsplit render.exr')

	 # removing the unused layers obtained from unused diffuse multi-layer 
     os.remove(dstRender)
     os.remove(exr_file_path + '/RenderLayer.Combined.exr')
	 
	 # re-naming the useful obtained from unused diffuse multi-layer 
     os.rename(exr_file_path + '/Combined.Combined.exr', exr_file_path + '/render.exr')
	 
     os.rename(exr_file_path + '/RenderLayer.Depth.exr', exr_file_path + '/depth.exr')
     os.rename(exr_file_path + '/RenderLayer.Normal.exr', exr_file_path + '/normal.exr')
	 
     os.rename(exr_file_path + '/RenderLayer.DiffCol.exr', exr_file_path + '/refl.exr')
     os.rename(exr_file_path + '/RenderLayer.DiffDir.exr', exr_file_path + '/diff_dir.exr')
     os.rename(exr_file_path + '/RenderLayer.DiffInd.exr', exr_file_path + '/diff_ind.exr')
     
	 # converting the intrinsic layers from .exr to .mat format 
     exrToMatCol(exr_file_path + '/render.exr', 0)
     exrToMatGray(exr_file_path + '/depth.exr')	 
     exrToMatCol(exr_file_path + '/normal.exr', 1)	 
     exrToMatCol(exr_file_path + '/refl.exr', 0)	 
     exrToMatCol(exr_file_path + '/diff_dir.exr', 0)	 
     exrToMatCol(exr_file_path + '/diff_ind.exr', 0)	
	 
	 # deleting the .exr files already saved as .mat files 
     os.remove(exr_file_path + '/render.exr')
     os.remove(exr_file_path + '/depth.exr')
     os.remove(exr_file_path + '/normal.exr')
     os.remove(exr_file_path + '/refl.exr')
     os.remove(exr_file_path + '/diff_dir.exr')
     os.remove(exr_file_path + '/diff_ind.exr')
	 
	 # un-comment the below code if you also want to save the LDR version of intrinsic layers 
     '''
     exrToLdrCol(exr_file_path + '/render.exr', orig_ldr_dir   + '/' + file + '.png', 0)
     exrToLdrGray(exr_file_path + '/depth.exr',  depth_ldr_dir  + '/' + file + '.png')	 
     exrToLdrCol(exr_file_path + '/normal.exr', normal_ldr_dir + '/' + file + '.png', 1)	 
     exrToLdrCol(exr_file_path + '/refl.exr', refl_ldr_dir   + '/' + file + '.png', 0)	 
     exrToLdrCol(exr_file_path + '/diff_dir.exr', shad_dir_ldr_dir + '/' + file + '.png', 0)	 
     exrToLdrCol(exr_file_path + '/diff_ind.exr', shad_ind_ldr_dir + '/' + file + '.png', 0)   
	 '''