# coding=utf-8
import os
from math import atan, copysign, cos, sin, sqrt, tan, degrees, pi, asin, acos
import mathutils


def get_initial_settings(scene):
    return scene.camera.location.copy(), scene.camera.rotation_euler.copy()


def calc_parallel(initial_location, initial_rotation, i):
    shift = i / 100

    new_location = initial_location.copy()
    new_location.x += shift * cos(initial_rotation.z)
    new_location.y += shift * sin(initial_rotation.z)

    return new_location, initial_rotation


def calc_parallel_rotate(initial_location, initial_rotation, i, x0, y0):
    shift = i / 100
    new_location = initial_location.copy()
    new_location.x += shift * cos(initial_rotation.z)
    new_location.y += shift * sin(initial_rotation.z)
    new_rotation = initial_rotation.copy()
    r = sqrt(pow(new_location.x - x0, 2) + pow(new_location.y - y0, 2))
    alpha = acos((new_location.y - y0) / r)
    if (new_location.x - x0) >= 0:  # if sin is positive than [0; pi)
        new_rotation.z = pi - alpha
    else:  # otherwise [pi; 2pi)
        new_rotation.z = alpha - pi

    return new_location, new_rotation


def calc_center_point(x, y, z, alpha, beta, r):
    x0 = x - r * cos(alpha - pi / 2) * cos(beta - pi / 2)
    y0 = y - r * cos(alpha - pi / 2) * sin(beta - pi / 2)
    z0 = z + r * sin(alpha - pi / 2)
    return x0, y0, z0

def argsToFloatArray(args):
    print(args)
    arr = []
    w = args.split(";")
    for x in range(0,len(w)):
        arr.append(float(w[x]))
    return arr

def argsToStringArray(args):
    print(args)
    arr = []
    w = args.split(";")
    for x in range(0,len(w)):
        arr.append(w[x])
    return arr


def main(camera_name, shift_distanceX, shift_distanceY, distance, out_dir, out_file_name,
         resolutionX, resolutionY, samples, scene_name):
    import bpy
    # import mathutils

    def save(scene_, filename=""):
        scene_.render.filepath = \
            os.path.join(out_dir, filename + scene_.render.file_extension)
        bpy.ops.render.render(write_still=True)
   

    scene = bpy.data.scenes[scene_name]
    bpy.context.screen.scene = scene

    scene.render.resolution_x = resolutionX
    scene.render.resolution_y = resolutionY
    scene.render.engine='CYCLES'
    scene.render.resolution_percentage = 100
    if samples:
        scene.cycles.samples = samples

    camera = bpy.data.cameras[camera_name]
    initial_camera_location, initial_camera_rotation = get_initial_settings(scene)
    inv = scene.camera.matrix_world.copy().to_3x3().normalized()
    #inv.invert()

    for i in range (0,len(shift_distanceX)):
        print("Shifting X by:", shift_distanceX[i])
        print("Shifting Y by:", shift_distanceY[i])
        vec = mathutils.Vector((shift_distanceX[i], shift_distanceY[i], 0.0))
        vec_trans = inv*vec
        #vec_trans = vec*inv
        scene.camera.location = initial_camera_location - vec_trans;
        #scene.render.image_settings.file_format = "HDR"
		#scene.render.image_settings.file_format = "OPEN_EXR_MULTILAYER"
        print("Saving:",out_file_name[i])
        save(scene, out_file_name[i])



if __name__ == "__main__":
    main(
        os.environ["CAMERANAME"],
        argsToFloatArray(os.environ["CAMERASHIFTX"]),
        argsToFloatArray(os.environ["CAMERASHIFTY"]),
        float(os.environ["DISTANCE"]),
        os.environ["OUTDIR"],
        argsToStringArray(os.environ["OUTFILENAME"]),
        float(os.environ.get("RESOLUTIONX")),
        float(os.environ.get("RESOLUTIONY")),
        int(os.environ["SAMPLES"]) if "SAMPLES" in os.environ else None,
        os.environ.get("SCENENAME", "Scene")
    )
