#!/usr/bin/python
import os, string, sys, os.path, math

ipfileName = "IPStatus.txt"

#What does this script do and how to use/troubleshoot it:
#
#This script creates jobs and batch files for distributed lightfield rendering for blender scenes
#The lightfields are generated by displacing otherwise identical cameras on a lightfield plane
#The cameras do not use a focal plane in this version of camera_shift_renderer
#
#Step by step tutorial:
#1. Place this file and "camera_shift_renderer.py" on the server, ideally in the same folder as your scene
#2. Setup Scene,Lightfield and Render settings in this file
#3. Run Angry IP scanner for the IP-adress range you want to distribute the jobs over and export all results to IPStatus.txt
#4. Place the IPStatus.txt in the same folder as this file
#5. Run this script (--> nohup python genJobs_lightfield.py). This will generate the jobs as well as "submitJobs.bat" and "killJobs.bat". In case you were running this on a windows machine, convert the jobs to linux style line endings!
#6. Copy the submitJobs.bat to your PC and search and replace USER and PSSWD with your username and password
#7. Run the submitJobs.bat file
#8. Optional: set resolution scaling and output format in "camera_shift_renderer.py"
#9. Check the logs for errors 
#
#Troubleshooting:
#Jobfiles/Rendered views are not written -> check the read/write settings of the respective folders
#Servers are not found when running submitJobs.bat -> make sure you are connected to the network via VPN when accessing from the outside and make sure you have permissions to use the compute cluster
#All outputs are black -> setting the output to a layered exr format did cause issues for me, hdr works
#Check logs and nohup.out for errors

#Scene settings
linuxWorkDirectory = "Rendered_Data/Living_Room"  
sceneLocation = "Rendered_Data/Living_Room/living_room.blend" # the location of blend file 
logsDirectory = "Rendered_Data/Living_Room/Logs" # the location of log files 
outdir = "Rendered"
jobsDirectoryName = "Jobs" # the location of job files 
blender = "Rendered_Data/blender/blender" # put a portable version of Blender (tested with ver 2.79) at this address 

#Lightfield settings
cameraShift = 0.04 #Distance between neighbouring cameras in the grid
distance = "3.2" #This used to be the distance of the focus plane, currently unused. Any code in the camera_shift_renderer using this may be interesting if you want to re-implement a focal plane
cameraArraySizeX = 11#Horizontal camera array resolution
cameraArraySizeY = 11 #Vertical camera array resolution
cameraArraySupersamplingFactor = 1 #Factor for the density of the Camera array; modifies the cameraArraySize and cameraShift while keeping the boundaries of the Camera-array the same. Use integers only!

#Render settings
cameraName = "Camera"
samples="2000"
resolutionX="1000.0"
resolutionY="1000.0"

#Supersample lightfield
cameraShift=cameraShift/float(cameraArraySupersamplingFactor)
cameraArraySizeX=cameraArraySizeX+(cameraArraySupersamplingFactor-1)*(cameraArraySizeX-1)
cameraArraySizeY=cameraArraySizeY+(cameraArraySupersamplingFactor-1)*(cameraArraySizeY-1)


dir = os.path.dirname(__file__)
jobsDirectory = os.path.join(dir, jobsDirectoryName)
renderDirectory = "//winfs-inf"+linuxWorkDirectory+"/"+outdir
keys = linuxWorkDirectory.split("/")
rkey = keys[len(keys)-1]

#amount of individual cameras to be rendered
jobsSize = float(cameraArraySizeX*cameraArraySizeY)


f1 = open('submitJobs.bat', 'w')
f2 = open('killJobs.bat','w')

#Getting working IPs
lineNum = 0
IPAddressArray = []
with open(ipfileName,"r") as f:
        for line in f:
                lineNum = lineNum + 1
                if (lineNum >= 8):
                        lineParts = line.split()
                        if((lineParts[1]=='[n/a]') | (lineParts[2]=='[n/s]')):
                                print("IP address in line "+str(lineNum)+" not usable")
                        else:
                                ipAddress = lineParts[2]

                                if "[n/s]" in ipAddress:
                                        ipAddress = ipAddress[:-5]
                                IPAddressArray.append(ipAddress)


								
machinesAvailable = float(len(IPAddressArray))
jobsPerMachine = math.ceil(jobsSize/machinesAvailable)
#distribute jobs the following way:
#Each machine is assigned jobsPerMachine many jobs until there are less jobs to be done than jobsPerMachine
#In that case, all remaining jobs are done by the next available machine and all remaining machines do NO JOBS!

#For each machine available to us:
for i in range (0,int(machinesAvailable)):

        #are there jobs remaining to be assigned to machines?
		if(i*jobsPerMachine>=jobsSize):
		    break #All jobs are distributed to machines
		
		#Jobs still need to be assigned to machines	
        #create job
		#Caution:Job-file contains ALL jobs assigned to the machine!
		jobFileName= jobsDirectory+'/JOB_'+str(i)
		jobF = open(jobFileName,'w')
		jobF.write("CAMERANAME=\""+cameraName+"\" \\\n")               

		xShiftString = "\""
		yShiftString = "\""
		outFileString= "\""

		for j in range(0,int(jobsPerMachine)):
				#Index of the camera in the lightfield to be assigned to the current job
				#The index starts at ZERO!
				currJobIndex = i*jobsPerMachine+j
                        
				#In case we run out of Jobs, there may be less than jobsPerMachine many jobs for the current machine 		
				if (currJobIndex<jobsSize):                              
						
						#Calculate x and y index of current camera in the lightfield
						yindex = math.floor(float(currJobIndex)/float(cameraArraySizeX))                        
						xindex = float(int(currJobIndex)-((cameraArraySizeX)*int(yindex)))
						
						#Shift the Camera in the scene according to its position in the lightfield
						#The Lightfield is centered around the original camera in the scene
						#X and Y shifts are in the normalized camera-coordinate system, not in world space!
						xshift = (float(cameraArraySizeX-1)/2.0 - xindex)* float(cameraShift)
						yshift = (yindex - float(cameraArraySizeY-1)/2.0)* float(cameraShift)
						
						outFileName = str(int(yindex))+"_"+str(int(xindex))                                        
						xShiftString = xShiftString + str(xshift)
						yShiftString = yShiftString + str(yshift)
						outFileString = outFileString + str(outFileName)
						if(j!=(jobsPerMachine-1)):
								xShiftString = xShiftString + ";"
								yShiftString = yShiftString + ";"
								outFileString = outFileString + ";"
		if(xShiftString.endswith(';')):
			xShiftString = xShiftString[:-1]
		if(yShiftString.endswith(';')):
			yShiftString = yShiftString[:-1]
		if(outFileString.endswith(';')):
			outFileString = outFileString[:-1]
		jobF.write("CAMERASHIFTX="+str(xShiftString)+"\" \\\n")
		jobF.write("CAMERASHIFTY="+str(yShiftString)+"\" \\\n")                        
		jobF.write("DISTANCE=\""+distance+"\" \\\n")
		jobF.write("OUTDIR=\""+outdir+"\" \\\n")
		jobF.write("OUTFILENAME="+outFileString+"\" \\\n")
		jobF.write("RESOLUTIONX=\""+resolutionX+"\" \\\n")
		jobF.write("RESOLUTIONY=\""+resolutionY+"\" \\\n")
		jobF.write("SAMPLES="+samples+" \\\n")

		jobF.write(blender+" \\\n")
		jobF.write("-b "+sceneLocation+" \\\n")
		jobF.write("--python "+linuxWorkDirectory+"/camera_shift_render.py\n")
          
		jobF.close()

		f1.write("plink.exe -ssh USER@"+IPAddressArray[i]+" -pw PSSWD \"cd "+linuxWorkDirectory+";chmod +x "+jobsDirectoryName+"/JOB_"+str(i)+";nohup ./"+jobsDirectoryName+"/JOB_"+str(i)+" > "+logsDirectory+"/"+rkey+"_"+str(i)+".txt 2>"+logsDirectory+"/"+rkey+"_"+str(i)+"_err.txt < /dev/null &\"\n")
		f2.write("plink.exe -ssh USER@"+IPAddressArray[i]+" -pw PSSWD \"pkill -9 blender\"\n")
f1.close()
f2.close()

